@@ -3,15 +3,13 @@ __all__ = ["Client", "YaqDaemonException"]
 
 import functools
 import inspect
-import io
 import json
-import socket
 from threading import Lock
 import types
 
-import fastavro  # type: ignore
-
 from ._socket import Socket
+from ._dotdict import DotDict
+from ._properties import Property
 
 
 class YaqDaemonException(Exception):
@@ -28,7 +26,8 @@ def reconnect(fun):
         try:
             return fun(self, *args, **kwargs)
         except ConnectionError:
-            self.connect_socket()
+            self._socket = Socket(self._host, self._port)
+            self.handshake()
             return fun(self, *args, **kwargs)
 
     return inner
@@ -40,6 +39,7 @@ class Client:
         self._port = port
         self._socket = Socket(self._host, self._port)
         self._id_counter = 0
+        self._connection_callbacks = []
         self._mutex = Lock()
         self.handshake()
 
@@ -77,9 +77,23 @@ class Client:
                 method.__doc__ = props.get("doc")
                 setattr(self, name, types.MethodType(method, self))
 
+            # Must be done after above message setattrs
+            self.properties = DotDict()
+            for k, v in self._protocol.get("properties", {}).items():
+                self.properties[k] = Property(self, v)
+            for cb in self._connection_callbacks:
+                cb()
+
+    @reconnect
     def send(self, method, *args, **kwargs):
         with self._mutex:
             self._id_counter += 1
             return self._socket.message(
                 method, self._protocol["messages"][method], *args, **kwargs
             )
+
+    def register_connection_callback(self, fun):
+        self._connection_callbacks.append(fun)
+
+    def clear_connection_callbacks(self):
+        self._connection_callbacks = []
