@@ -1,11 +1,11 @@
 __all__ = ["Socket"]
 
 
-import fastavro  # type: ignore
 import socket
 import io
 import struct
 
+import fastavro  # type: ignore
 
 from ._schema import handshake_request, handshake_response
 
@@ -16,10 +16,18 @@ BUFFSIZE = 4096
 class Socket:
     def __init__(self, host, port):
         self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        self._socket.settimeout(1)
+        self._socket.settimeout(None)
         self._socket.connect((host, port))
+        self._named_types = {}
 
     def _read(self, response_schema):
+        response_schema = fastavro.parse_schema(
+            response_schema, expand=True, named_schemas=self._named_types
+        )
+        # Needed twice for nested types, likely should be fixed upstream
+        response_schema = fastavro.parse_schema(
+            response_schema, expand=True, named_schemas=self._named_types
+        )
         buf = io.BytesIO()
         remaining = 0
         while True:
@@ -27,9 +35,8 @@ class Socket:
                 buf.seek(0)
                 obj = fastavro.schemaless_reader(buf, response_schema)
                 return obj
-            except Exception as e:
+            except Exception:
                 buf.seek(0)
-                pass
             if not remaining:
                 remaining = struct.unpack_from(">L", self._socket.recv(4))[0]
 
@@ -43,7 +50,9 @@ class Socket:
         out = struct.pack(">L", len(out)) + out
         self._socket.sendall(out)
 
-    def handshake(self, client_hash=b" " * 16, client_protocol=None, server_hash=b" " * 16):
+    def handshake(
+        self, client_hash=b" " * 16, client_protocol=None, server_hash=b" " * 16
+    ):
         # send request
         request = io.BytesIO()
         record = {
@@ -63,7 +72,9 @@ class Socket:
         self._read("null")
         if response["match"] == "NONE":
             self.handshake(
-                response["serverHash"], response["serverProtocol"], response["serverHash"],
+                response["serverHash"],
+                response["serverProtocol"],
+                response["serverHash"],
             )
         return response["serverProtocol"]
 
@@ -87,7 +98,9 @@ class Socket:
             meta = {}
         # write metadata
         out = io.BytesIO()
-        fastavro.schemaless_writer(out, {"type": "map", "values": "bytes"}, meta)  # empty mapping
+        fastavro.schemaless_writer(
+            out, {"type": "map", "values": "bytes"}, meta
+        )  # empty mapping
         self._write(out)
 
     def _write_method_name(self, method_name):
@@ -105,7 +118,14 @@ class Socket:
             elif args:
                 data = args.pop(0)
             out = io.BytesIO()
-            fastavro.schemaless_writer(out, parameter["type"], data)
+            schema = fastavro.parse_schema(
+                parameter["type"], expand=True, named_schemas=self._named_types
+            )
+            # Needed twice for nested types... Should likely be fixed upstream
+            schema = fastavro.parse_schema(
+                schema, expand=True, named_schemas=self._named_types
+            )
+            fastavro.schemaless_writer(out, schema, data)
             self._write(out)
 
     def _write_terminator(self):
